@@ -8,24 +8,24 @@ use std::net::SocketAddr;
 use std::net::TcpListener;
 use std::net::TcpStream;
 use std::str;
+use std::time::Duration;
 
 // Feature Imports
 #[cfg(feature = "panic_handler")]
 use std::panic;
 
-#[cfg(feature = "cookies")]
-use super::cookie::Cookie;
-
 #[cfg(feature = "thread_pool")]
 use super::threadpool::ThreadPool;
 
 // Import local files
+use super::common::reason_phrase;
 use super::header::{headers_to_string, Header};
+use super::http;
 use super::method::Method;
-use super::query::Query;
 use super::request::Request;
 use super::response::Response;
 use super::route::Route;
+use super::VERSION;
 
 /// Default Buffer Size
 ///
@@ -55,9 +55,13 @@ pub struct Server {
 
     /// Default response for internal server errors
     #[cfg(feature = "panic_handler")]
-    error_handler: fn(Request) -> Response,
+    error_handler: fn(Request, String) -> Response,
+
     /// Headers automatically added to every response.
     default_headers: Option<Vec<Header>>,
+
+    /// Socket Timeout
+    socket_timeout: Option<Duration>,
 }
 
 /// Implementations for Server
@@ -99,14 +103,15 @@ impl Server {
             middleware: Vec::new(),
             run: true,
             #[cfg(feature = "panic_handler")]
-            error_handler: |_| {
+            error_handler: |_, err| {
                 Response::new(
                     500,
-                    "Internal Server Error :/",
+                    &format!("Internal Server Error :/\n{}", err),
                     vec![Header::new("Content-Type", "text/plain")],
                 )
             },
-            default_headers: Some(vec![Header::new("Server", "afire")]),
+            default_headers: Some(vec![Header::new("Server", &format!("afire/{}", VERSION))]),
+            socket_timeout: None,
         }
     }
 
@@ -135,19 +140,21 @@ impl Server {
     /// // This is blocking
     /// # // Keep the server from starting and blocking the main thread
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
-    pub fn start(&self) {
+    pub fn start(&self) -> Option<()> {
         // Exit if the server should not run
         if !self.run {
-            return;
+            return Some(());
         }
 
-        let listener = init_listener(self.ip, self.port).unwrap();
+        let listener = init_listener(self.ip, self.port).ok()?;
 
         for event in listener.incoming() {
             // Read stream into buffer
-            let mut stream = event.unwrap();
+            let mut stream = event.ok()?;
+            stream.set_read_timeout(self.socket_timeout).unwrap();
+            stream.set_write_timeout(self.socket_timeout).unwrap();
 
             // Get the response from the handler
             // Uses the most recently defined route that matches the request
@@ -163,8 +170,9 @@ impl Server {
             // Convert the response to a string
             // TODO: Use Bytes instead of String
             let mut response = format!(
-                "HTTP/1.1 {} OK\r\n{}\r\n\r\n",
+                "HTTP/1.1 {} {}\r\n{}\r\n\r\n",
                 res.status,
+                reason_phrase(res.status),
                 headers_to_string(headers)
             )
             .as_bytes()
@@ -174,12 +182,20 @@ impl Server {
             response.append(&mut res.data);
 
             // Send the response
-            stream.write_all(&response).unwrap();
+            let _ = stream.write_all(&response);
             stream.flush().unwrap();
         }
+
+        // We should Never Get Here
+        None
     }
 
     /// Start the server with a thread pool.
+    ///
+    /// **IN DEVELOPMENT**
+    ///
+    /// Currently will not work with any middleware.
+    /// Everything else works though
     /// ## Example
     /// ```rust
     /// // Import Library
@@ -204,10 +220,10 @@ impl Server {
     /// server.start_threaded(8);
     /// ```
     #[cfg(feature = "thread_pool")]
-    pub fn start_threaded(&self, threads: usize) {
+    pub fn start_threaded(&self, threads: usize) -> Option<()> {
         // Exit if the server should not run
         if !self.run {
-            return;
+            return Some(());
         }
 
         let listener = init_listener(self.ip, self.port).unwrap();
@@ -215,11 +231,13 @@ impl Server {
 
         for event in listener.incoming() {
             // Read stream into buffer
-            let stream = event.unwrap();
+            let stream = event.ok()?;
+            stream.set_read_timeout(self.socket_timeout).unwrap();
+            stream.set_write_timeout(self.socket_timeout).unwrap();
 
             let routes = self.routes.clone();
-            // let middleware = self.middleware.clone();
             let error_handler = self.error_handler;
+            let default_headers = self.default_headers.clone();
 
             pool.execute(move || {
                 let mut stream = stream;
@@ -229,7 +247,7 @@ impl Server {
 
                 // Add default headers to response
                 let mut headers = res.headers;
-                // headers.append(&mut self.default_headers.clone().unwrap_or_default());
+                headers.append(&mut default_headers.unwrap_or_default());
 
                 // Add content-length header to response
                 headers.push(Header::new("Content-Length", &res.data.len().to_string()));
@@ -237,8 +255,9 @@ impl Server {
                 // Convert the response to a string
 
                 let mut response = format!(
-                    "HTTP/1.1 {} OK\r\n{}\r\n\r\n",
+                    "HTTP/1.1 {} {}\r\n{}\r\n\r\n",
                     res.status,
+                    reason_phrase(res.status),
                     headers_to_string(headers)
                 )
                 .as_bytes()
@@ -248,10 +267,13 @@ impl Server {
                 response.append(&mut res.data);
 
                 // Send the response
-                stream.write_all(&response).unwrap();
+                let _ = stream.write_all(&response);
                 stream.flush().unwrap();
             });
         }
+
+        // Again we should never get here
+        None
     }
 
     fn handle_connection(&self, stream: &TcpStream) -> Response {
@@ -276,13 +298,12 @@ impl Server {
     /// server.set_run(false);
     ///
     /// // 'Start' the server
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
     pub fn set_run(&mut self, run: bool) {
         self.run = run;
     }
 
-    #[cfg(feature = "panic_handler")]
     /// Set the panic handler response
     ///
     /// Default response is 500 "Internal Server Error :/"
@@ -299,15 +320,16 @@ impl Server {
     /// let mut server: Server = Server::new("localhost", 8080);
     ///
     /// // Set the panic handler response
-    /// server.set_error_handler(|_req| {
-    ///    Response::new(500, "Internal Server Error", vec![])
+    /// server.set_error_handler(|_req, err| {
+    ///    Response::new(500, &format!("Internal Server Error: {}", err), vec![])
     /// });
     ///
     /// // Start the server
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
-    pub fn set_error_handler(&mut self, res: fn(Request) -> Response) {
+    #[cfg(feature = "panic_handler")]
+    pub fn set_error_handler(&mut self, res: fn(Request, String) -> Response) {
         self.error_handler = res;
     }
 
@@ -347,7 +369,7 @@ impl Server {
     /// // Start the server
     /// // As always, this is blocking
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
     pub fn add_default_header(&mut self, header: Header) {
         self.default_headers
@@ -356,6 +378,29 @@ impl Server {
             .push(header);
     }
 
+    /// Set the socket Read / Write Timeout
+    ///
+    /// ## Example
+    /// ```rust
+    /// // Import Library
+    /// use std::time::Duration;
+    /// use afire::Server;
+    ///
+    /// // Create a server for localhost on port 8080
+    /// let mut server: Server = Server::new("localhost", 8080);
+    ///
+    /// // Set socket timeout
+    /// server.set_socket_timeout(Some(Duration::from_secs(1)));
+    ///
+    /// // Start the server
+    /// // As always, this is blocking
+    /// # server.set_run(false);
+    /// server.start().unwrap();
+    /// ```
+    pub fn set_socket_timeout(&mut self, socket_timeout: Option<Duration>) {
+        self.socket_timeout = socket_timeout;
+    }
+
     /// Create a new route the runs for all methods and paths
     ///
     /// May be useful for a 404 page as the most recently defined route takes priority
@@ -391,7 +436,7 @@ impl Server {
     /// // Starts the server
     /// // This is blocking
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
     pub fn all(&mut self, handler: fn(Request) -> Response) {
         self.routes
@@ -419,7 +464,7 @@ impl Server {
     /// // Starts the server
     /// // This is blocking
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
     /// Now you can make any type of request to `/nose` and it will return a 200
     #[deprecated(since = "0.1.5", note = "Instead use .route(Method::ANY...)")]
@@ -453,7 +498,7 @@ impl Server {
     /// // Starts the server
     /// // This is blocking
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
     pub fn every(&mut self, handler: Box<dyn Fn(&Request) -> Option<Response>>) {
         self.middleware.push(handler);
@@ -480,7 +525,7 @@ impl Server {
     /// // Starts the server
     /// // This is blocking
     /// # server.set_run(false);
-    /// server.start();
+    /// server.start().unwrap();
     /// ```
     pub fn route(&mut self, method: Method, path: &str, handler: fn(Request) -> Response) {
         self.routes
@@ -492,50 +537,60 @@ impl Server {
 fn handle_connection(
     mut stream: &TcpStream,
     middleware: &[Box<dyn Fn(&Request) -> Option<Response>>],
-    error_handler: fn(Request) -> Response,
+    error_handler: fn(Request, String) -> Response,
     routes: &[Route],
 ) -> Response {
     // Init (first) Buffer
     let mut buffer = vec![0; BUFF_SIZE];
 
     // Read stream into buffer
-    let _ = stream.read(&mut buffer).unwrap();
+    match stream.read(&mut buffer) {
+        Ok(_) => {}
+        Err(_) => return quick_err("Error Reading Stream", 500),
+    };
 
     // Get buffer as string
     let buffer_clone = buffer.clone();
     let stream_string = match str::from_utf8(&buffer_clone) {
         Ok(s) => s,
-        Err(_) => return Response::new(500, "Internal Server Error", vec![]),
+        Err(_) => return quick_err("Internal Server Error", 500),
     };
 
     // Get Content-Length header
     // If header shows thar more space is needed,
     // make a new buffer read the rest of the stream and add it to the first buffer
-    for i in get_request_headers(stream_string.to_string()) {
+    for i in http::get_request_headers(stream_string.to_string()) {
         if i.name != "Content-Length" {
             continue;
         }
-        let header_size = get_header_size(stream_string.to_string());
+        let header_size = http::get_header_size(stream_string.to_string());
         let content_length = i.value.parse::<usize>().unwrap_or(0);
         let new_buffer_size = content_length as i64 + header_size as i64 - BUFF_SIZE as i64;
         if new_buffer_size > 0 {
             let mut new_buffer = vec![0; new_buffer_size as usize];
-            let _ = stream.read(&mut new_buffer).unwrap();
+            match stream.read(&mut new_buffer) {
+                Ok(_) => {}
+                Err(_) => return quick_err("Error Reading Stream", 500),
+            };
             buffer.append(&mut new_buffer);
         }
         break;
     }
 
-    let stream_string = str::from_utf8(&buffer).expect("Error parsing buffer data");
+    // TODO: Make this work with non utf8 stuff too
+    let stream_string = match str::from_utf8(&buffer) {
+        Ok(i) => i,
+        Err(_) => return quick_err("No support for non utf-8 chars\nFor now", 500),
+    };
 
     // Make Request Object
-    let req_method = get_request_method(stream_string.to_string());
-    let req_path = get_request_path(stream_string.to_string());
-    let req_query = get_request_query(stream_string.to_string());
-    let body = get_request_body(stream_string.to_string());
-    let headers = get_request_headers(stream_string.to_string());
+    let req_method = http::get_request_method(stream_string.to_string());
+    let req_path = http::get_request_path(stream_string.to_string());
+    let req_query = http::get_request_query(stream_string.to_string());
+    let body = http::get_request_body(stream_string.to_string());
+    let headers = http::get_request_headers(stream_string.to_string());
     #[cfg(feature = "cookies")]
-    let cookies = get_request_cookies(stream_string.to_string());
+    let cookies = http::get_request_cookies(stream_string.to_string());
     let req = Request::new(
         req_method,
         &req_path,
@@ -566,7 +621,14 @@ fn handle_connection(
             #[cfg(feature = "panic_handler")]
             {
                 let result = panic::catch_unwind(|| (route.handler)(req.clone()));
-                return result.ok().unwrap_or_else(|| (error_handler)(req));
+                let err = match result {
+                    Ok(i) => return i,
+                    Err(e) => match e.downcast_ref::<&str>() {
+                        Some(err) => err,
+                        None => "",
+                    },
+                };
+                return (error_handler)(req, err.to_string());
             }
 
             #[cfg(not(feature = "panic_handler"))]
@@ -592,99 +654,7 @@ fn init_listener(ip: [u8; 4], port: u16) -> Result<TcpListener, io::Error> {
     ))
 }
 
-/// Get the request method of a raw HTTP request.
-fn get_request_method(raw_data: String) -> Method {
-    let method_str = raw_data
-        .split(' ')
-        .collect::<Vec<&str>>()
-        .get(0)
-        .unwrap()
-        .to_string();
-
-    match &method_str[..] {
-        "GET" => Method::GET,
-        "POST" => Method::POST,
-        "PUT" => Method::PUT,
-        "DELETE" => Method::DELETE,
-        "OPTIONS" => Method::OPTIONS,
-        "HEAD" => Method::HEAD,
-        "PATCH" => Method::PATCH,
-        "TRACE" => Method::TRACE,
-        _ => Method::CUSTOM(method_str),
-    }
-}
-
-/// Get the path of a raw HTTP request.
-fn get_request_path(raw_data: String) -> String {
-    let path_str = raw_data.split(' ').collect::<Vec<&str>>();
-    if path_str.len() > 1 {
-        let path = path_str[1].to_string();
-        let path = path.split('?').collect::<Vec<&str>>();
-        return path[0].to_string();
-    }
-    "".to_string()
-}
-
-// Get The Query Data of a raw HTTP request.
-fn get_request_query(raw_data: String) -> Query {
-    let path_str = raw_data.split(' ').collect::<Vec<&str>>();
-    if path_str.len() > 1 {
-        let path = path_str[1].to_string();
-        let path = path.split('?').collect::<Vec<&str>>();
-
-        if path.len() <= 1 {
-            return Query::new_empty();
-        }
-        return Query::new(path[1]);
-    }
-    Query::new_empty()
-}
-
-/// Get the body of a raw HTTP request.
-fn get_request_body(raw_data: String) -> String {
-    let data = raw_data.split("\r\n\r\n").collect::<Vec<&str>>();
-
-    if data.len() >= 2 {
-        return data[1].to_string().trim_matches(char::from(0)).to_string();
-    }
-    "".to_string()
-}
-
-/// Get the headers of a raw HTTP request.
-fn get_request_headers(raw_data: String) -> Vec<Header> {
-    let mut headers = Vec::new();
-    let spilt = raw_data.split("\r\n\r\n").collect::<Vec<&str>>();
-    let raw_headers = spilt[0].split("\r\n").collect::<Vec<&str>>();
-
-    for header in raw_headers {
-        if let Some(header) = Header::from_string(header.trim_matches(char::from(0))) {
-            headers.push(header)
-        }
-    }
-
-    headers
-}
-
-/// Get Cookies of a raw HTTP request.
-#[cfg(feature = "cookies")]
-pub fn get_request_cookies(raw_data: String) -> Vec<Cookie> {
-    let spilt = raw_data.split("\r\n\r\n").collect::<Vec<&str>>();
-    let raw_headers = spilt[0].split("\r\n").collect::<Vec<&str>>();
-
-    for header in raw_headers {
-        if !header.starts_with("Cookie:") {
-            continue;
-        }
-
-        if let Some(cookie) = Cookie::from_string(header.trim_matches(char::from(0))) {
-            return cookie;
-        }
-    }
-    Vec::new()
-}
-
-/// Get the byte size of the headers of a raw HTTP request.
-fn get_header_size(raw_data: String) -> usize {
-    let headers = raw_data.split("\r\n\r\n").collect::<Vec<&str>>();
-    headers[0].len() + 4
+/// Quick function to get a basic error response
+fn quick_err(text: &str, code: u16) -> Response {
+    Response::new(code, text, vec![Header::new("Content-Type", "text/plain")])
 }
